#include <stdlib.h>
#include "Types.h"
#include "testlib.h"
#include "EIDIO.h"

uint32 PCIeIDIO24_ReadBackAll( Word a ) {return InPortDW( a + 0x00 );}
uint32 PCIeIDIO24_ReadBackISO( Word a ) {return InPortDW( a + 0x00 ) & 0x00FFFFFFL;}

Byte PCIeIDIO24_ReadBackTTL( Word a )
{
    return ( InByte( a + 0x03 ) );
}
void PCIeIDIO24_WriteAll( Word a, uint32 d )
{
    OutPortDW( a + 0x00, d );
}

void PCIeIDIO24_WriteISO( Word a, uint32 d )
{
    OutPortDW( a + 0x00, ( d & 0x00FFFFFFL ) | ( ( uint32 ) PCIeIDIO24_ReadBackTTL( a ) << 24 ) );
}   // PCIeIDIO24_WriteISO

void PCIeIDIO24_WriteTTL( Word a, Byte d )
{
    OutByte( a + 0x03, d );
}

Byte PCIeIDIO24_ReadTTL( Word a )
{
    return ( InByte( a + 0x07 ) );
}

uint32 PCIeIDIO24_ReadAll( Word a )
{
    return ( InPortDW( a + 0x04 ) );
}

uint32 PCIeIDIO24_ReadISO( Word a )
{
    return ( InPortDW( a + 0x04 ) & 0x00FFFFFFL );
}

uint32 PCIeIDIO24_COSStatus( Word a )
{
    return ( InPortDW( a + 0x08 ) );    // "set" bits mean that bit generated a COS
}

void PCIeIDIO24_ClearCOSBits( Word a, uint32 COSBitsToClear )
{
    OutPortDW( a + 0x08, COSBitsToClear );  // pass "1" in each bit to clear that bit
}

uint32 PCIeIDIO24_ClearAndCheckCOSBits( Word a, uint32 COSBitsToClear )
{
    PCIeIDIO24_ClearCOSBits( a, COSBitsToClear );
    return ( PCIeIDIO24_COSStatus( a ) );
}

// =============================================================================================================================================================
//    controls for base + C
// =============================================================================================================================================================
void PCIeIDIO24_EnableTTLOutputBuffer( Word a )
{
    OutByte( a + 0x0C, ( InByte( a + 0x0C ) & 0x0F ) | 0x01 );
}

void PCIeIDIO24_DisableTTLOutputBuffer( Word a )
{
    OutByte( a + 0x0C, InByte( a + 0x0C ) &~0xF1 );
}

void PCIeIDIO24_SetTTLOut( Word a )
{
    OutByte( a + 0x0C, ( InByte( a + 0x0C ) & 0x0F ) | 0x02 );
}

Byte PCIeIDIO24_SetTTLIn( Word a )
{
    OutByte( a + 0x0C, ( InByte( a + 0x0C ) &~0xF2 ) );
    return ( InByte( a + 0x0c ) );
}

void PCIeIDIO24_EnableFilter( Word a, bool zero11, bool twelve23 )
{
    OutByte( a + 0x0C, ( InByte( a + 0x0C ) & 0x03 ) | ( zero11 ? 0x04 : 0x00 ) | ( twelve23 ? 0x08 : 0x00 ) );
}

// =============================================================================================================================================================
//    controls for base + E
// =============================================================================================================================================================
void PCIeIDIO24_EnableCOS
(
    Word  Base,
    bool    bRising0to7,
    bool    bRising8to15,
    bool    bRising16to23,
    bool    RisingTTL,
    bool    bFalling0to7,
    bool    bFalling8to15,
    bool    bFalling16to24,
    bool    FallingTTL
)
{
    OutByte
    (
        Base + 0x0E,
        ( bRising0to7 ? 0x01 : 0x00 ) | ( bRising8to15 ? 0x02 : 0x00 ) | ( bRising16to23 ? 0x04 : 0x00 ) | ( RisingTTL ? 0x08 : 0x00 ) | ( bFalling0to7 ? 0x10 : 0x00 ) |
        ( bFalling8to15 ? 0x20 : 0x00 ) | ( bFalling16to24 ? 0x40 : 0x00 ) | ( FallingTTL ? 0x80 : 0x00 )
    );
}

void PCIeIDIO24_EnableCOS( Word Base, Byte COSEnableBits )
{
    OutByte( Base + 0x0E, COSEnableBits );
}

// =============================================================================================================================================================
//    sets all registers other than OEN to 1, sets OEN to zero (outputs disabled, input mode, no COS enabled)
// =============================================================================================================================================================
void PCIeIDIO24_SoftReset( Word Base )
{
    OutByte( Base + 0x0F, 0 );
}

uint32 PCIeIDIO24_GenerateWalkAll( Byte state )
{
    return ( bitwalk( state, 8 ) << 24 ) | ( bitwalk( state, 24 ) );
}

uint32 PCIeIDIO24_WalkAll( Word a, Byte state )
{
    uint32  t = PCIeIDIO24_GenerateWalkAll( state );
    //----------------------------------------------
    PCIeIDIO24_WriteISO( a + 0x00, t );
    return ( t );
}

bool FilteredCompare32( uint32 a, uint32 b, uint32 filter )
{
    return ( !( ( a ^ b ) & filter ) );
}

uint32 PCIeIDIO24_wrapplugmapfunc( uint32 input )
{
    dwordX  src, dst;
    //---------------
    src.value32 = input;
    dst.value32 = 0;
    dst.nyb.n0 = src.nyb.n0;
    dst.bit.d04 = src.bit.d04;
    dst.bit.d05 = src.bit.d05;
    dst.bit.d16 = src.bit.d16;
    dst.bit.d17 = src.bit.d17;
    dst.nyb.n3 = src.nyb.n3;
    dst.nyb.n2 = src.nyb.n2;
    dst.bit.d06 = src.bit.d06;
    dst.bit.d07 = src.bit.d07;
    dst.bit.d18 = src.bit.d18;
    dst.bit.d19 = src.bit.d19;
    dst.nyb.n5 = src.nyb.n5;
    dst.byt.b3 = src.byt.b3;
    return ( dst.value32 );
}   // PCIeIDIO24_wrapplugmapfunc

uint32 original_PCIeIDIO24_wrapplugmapfunc( uint32 input )
{
    dwordX  src, dst;
    //---------------
    src.value32 = input;
    dst.value32 = 0;
    dst.nyb.n0 = src.nyb.n0;
    dst.bit.d04 = src.bit.d04;
    dst.bit.d05 = src.bit.d05;
    dst.bit.d06 = src.bit.d16;
    dst.bit.d07 = src.bit.d17;
    dst.nyb.n2 = src.nyb.n3;
    dst.nyb.n3 = src.nyb.n2;
    dst.bit.d16 = src.bit.d06;
    dst.bit.d17 = src.bit.d07;
    dst.bit.d18 = src.bit.d18;
    dst.bit.d19 = src.bit.d19;
    dst.nyb.n5 = src.nyb.n5;
    dst.byt.b3 = src.byt.b3;
    return ( dst.value32 );
}   // PCIeIDIO24_wrapplugmapfunc

uint32  PCIeIDIO24_IRQBits;
uint32  PCIeIDIO24_IRQData;

void interrupt far PCIeIDIO24_ISR( ... )
{
    IRQCount++;
    PCIeIDIO24_IRQData |= PCIeIDIO24_ClearAndCheckCOSBits( testRun.theCard.Base, PCIeIDIO24_IRQBits );
    PCIeIDIO24_ClearCOSBits( testRun.theCard.Base, 0xFFFFFFFFL );
    sendEOI();
}

uint32 PCIeIDIO24_CoSInner( char* msg, Byte COSMask, Byte bDirection, bool bIRQExpected )
{
    uint32  m = 0, isum = 0, err = 0;
    Byte  y = 0, x = 0;
    uint32  t = bDirection ? 0x00000000L : 0xFFFFFFFFL;
    uint32  irqsum = 0;
    uint32  irqsuccess = 0;
    uint32  irqfails = 0;
    char    smsg[256] = "";

    InitISR( PCIeIDIO24_ISR );
    SubTestAnnounce( msg );
    y = wherey();
    x = wherex();
    PCIeIDIO24_EnableCOS( testRun.theCard.Base, COSMask );
    PCIeIDIO24_WriteAll( testRun.theCard.Base, t );

    for ( Byte i = 0; ( i < 32 ) && ( !kbhit() ); i++ )
    {
        m = bitwalk( i, 32 );

        if ( bDirection ) { t |= m; }
        else { t &= ~m; }

        PCIeIDIO24_IRQBits = PCIeIDIO24_wrapplugmapfunc( m );
        PCIeIDIO24_WriteAll( testRun.theCard.Base, t );
        delay( 1 );
        irqsum += IRQCount;
        sprintf( smsg, "Expect %2hd/%2hd IRQs, Got %2ld  ", ( bIRQExpected ? i + 1 : 0 ), i + 1, irqsum );
        gotoxy( x, y );
        cprintf( "%39s", smsg );

        if ( IRQCount != ( bIRQExpected ? 1 : 0 ) )
        {
            irqfails++;
            err |= PCIeIDIO24_IRQBits;
            cprintb( err, 32, TRUE );
            NewLine();
        }
        else
        {
            irqsuccess++;
        }

        IRQCount = 0;
        isum |= PCIeIDIO24_IRQData;
        PCIeIDIO24_IRQData = 0L;
    }   // end for loop

    if ( kbhit() )
    {

        getch();
        gotoxy( x, y );
        clreol();
        textcolor( DARKGRAY );
    }

    if (testRun.bDebug) Delay(DELAY_Pacing);

    PCIeIDIO24_WriteAll( testRun.theCard.Base, bDirection ? 0xFFFFFFFFL : 0x00000000L );
    deInitISR();

    return SubTestResult( err );
}   // PCIeIDIO24_CoSInner

void PCIeIDIO24_CoSTest()
{
    uint32  sumerr = 0, err = 0;
    uint32  t = 0;
    Byte  y, i;
    Word diobase = testRun.theCard.Base;

    TestAnnounce( "Change-of-State Test" );

    IRQCount = 0;
    PCIeIDIO24_SoftReset( diobase );
    PCIeIDIO24_SetTTLOut( diobase );
    InitISR( PCIeIDIO24_ISR );

    delay( 1 );

    sumerr += PCIeIDIO24_CoSInner( "No Rising COS when disabled",              0x00, 1, FALSE );
    sumerr += PCIeIDIO24_CoSInner( "No Falling COS when disabled",             0x00, 0, FALSE );
    sumerr += PCIeIDIO24_CoSInner( "No Rising COS when only falling Enabled",  0xF0, 1, FALSE );
    sumerr += PCIeIDIO24_CoSInner( "No Falling COS when only rising Enabled",  0x0F, 0, FALSE );

    sumerr += PCIeIDIO24_CoSInner( "Rising COS with only rising Enabled",      0x0F, 1, TRUE );
    sumerr += PCIeIDIO24_CoSInner( "Falling COS with only falling Enabled",    0xF0, 0, TRUE );
    sumerr += PCIeIDIO24_CoSInner( "Rising COS with both directions Enabled",  0xFF, 1, TRUE );
    sumerr += PCIeIDIO24_CoSInner( "Falling COS with both dirs Enabled", 0xFF, 0, TRUE );

    TestResult( sumerr ?1:0 );

    //    PCIeIDIO24_SoftReset( diobase );
}   // PCIeIDIO24_CoSTest

void doIrqPlayground( void )
{
    TestAnnounce("IRQ playground");
    Instruct("The number of IRQs received will be printed onscreen.");
    Instruct("Press ESC to exit the Playground");
    Instruct("If you connect one or more FETs to the Inputs, pressing\r\n"
             "any other key will write a random value to the FET and TTL bits\r\n"
             "If you only have one FET looped back to an input, you may have\r\n"
             "to hit a key several times to see an IRQ"
            );
    uint32  m = 0, isum = 0, err = 0;
    Byte  y = 0, x = 0;
    uint32  irqsum = 0;
    uint32  irqsuccess = 0;
    uint32  irqfails = 0;
    char    smsg[256] = "";
    char key=0;

    InitISR( PCIeIDIO24_ISR );
    y = wherey();
    x = wherex();
    PCIeIDIO24_EnableCOS( testRun.theCard.Base, 0xFF );// enable all IRQ sources

    while (TRUE)
    {
       irqsum += IRQCount;
       sprintf( smsg, "Got %2ld IRQs", irqsum );
       gotoxy( x, y );
       cprintf( "%39s", smsg );

       IRQCount = 0;
       isum |= PCIeIDIO24_IRQData;
       PCIeIDIO24_IRQData = 0L;

       if ( kbhit() )
       {
           key=getch();
           if (key==27) break;
           PCIeIDIO24_WriteAll( testRun.theCard.Base, rand() );
           gotoxy( x, y );
           clreol();
           textcolor( DARKGRAY );
           cprintf( "%30s", "Skipped by user" );
       }
    }
    deInitISR();
    TestResult(irqsum==0);
}

void doPCIeIDIO24( void )
{
    InstructAndWait( "Ensure test cable is attached" );

    PCIeIDIO24_CoSTest( );
    doIrqPlayground();
}   // doPCIeIDIO24

