#include <string.h>
#include "TestLib.h"

Word IRQCount = 0;
TpISR oldisr;
TpISR test_specific_irq_handler;

TISR_Handler hISR;

Word IRQConfigAddr, IRQClearAddr;

void initirq( int IRQnumber, void far interrupt( *ISR )( ... ) )
{
    Byte  intmask, oldmask;

    if ( IRQnumber <= 7 )
    {
        oldmask = InByte( 0x21 );
        oldisr = getvect( IRQnumber + 8 );
        setvect( IRQnumber + 8, ISR );
        intmask = oldmask & ( ~( 1 << IRQnumber ) );
        OutByte( 0x21, intmask );
    }
    else
    {
        oldmask = InByte( 0xA1 );
        oldisr = getvect( IRQnumber + 0x70 - 8 );
        setvect( IRQnumber - 8 + 0x70, ISR );
        intmask = oldmask & ( ~( 1 << ( IRQnumber - 8 ) ) );
        OutByte( 0xA1, intmask );
    }
    sendEOI();
}

TISR_Handler InitISR( Word Base, int IRQNumber, Taction clearAction, int clearOffset, Word &clearValue, Word* pIRQFlag, TpISR anISR )
{
    hISR.Base = Base;
    hISR.Status = 0;
    hISR.bDone = TRUE;
    hISR.IRQ = IRQNumber;
    hISR.IRQClear.Action = clearAction;
    hISR.IRQClear.Offset = clearOffset;
    hISR.IRQClear.Value = clearValue;

    hISR.pIRQFlag = pIRQFlag ? pIRQFlag : &IRQCount;
    *hISR.pIRQFlag = 0;

    hISR.ISR = anISR;

    initirq( hISR.IRQ, hISR.ISR );

    return hISR;
};


TISR_Handler InitISR( TRegAct &clearAction, Word* pIRQFlag, TpISR anISR)
{
  return InitISR( testRun.theCard.Base, testRun.theCard.IRQ, clearAction.Action, clearAction.Offset, clearAction.Value, pIRQFlag, anISR );
}

TISR_Handler InitISR( Word Base, int IRQ, TRegAct act )
{
  return InitISR( Base, IRQ, act.Action, act.Offset, act.Value, NULL );
}

TISR_Handler InitISR( TRegAct act )
{
  return InitISR( testRun.theCard.Base, testRun.theCard.IRQ, act.Action, act.Offset, act.Value, NULL );
}

TISR_Handler InitISR( Word Base, int IRQNumber, TpISR anISR )
{
    return InitISR( Base, IRQNumber, NotApplicable, 0, 0, NULL, anISR );
}

TISR_Handler InitISR( TpISR anISR )
{
    return InitISR( testRun.theCard.Base, testRun.theCard.IRQ, anISR );
}

void sendEOI( void )
{
    OutByte( 0x20, 0x20 ); // send non-specific to master AND slave PIC
    OutByte( 0xA0, 0x20 );
}

void restoreirq( Byte IRQnumber )
{
    Byte  intmask;

    if ( IRQnumber <= 7 )
    {
        intmask = InByte( 0x21 );
        intmask |= ( 1 << IRQnumber );
        setvect( IRQnumber + 8, oldisr );
        OutByte( 0x21, intmask );
    }
    else
    {
        intmask = InByte( 0xA1 );
        intmask |= ( 1 << ( IRQnumber - 8 ) );
        setvect( IRQnumber - 8 + 0x70, oldisr );
        OutByte( 0xA1, intmask );
    }
}   // restoreirq


void deInitISR( Byte IRQ )
{   restoreirq( IRQ ); }

void deInitISR( TISR_Handler ISR )
{   restoreirq( ISR.IRQ ); }

void deInitISR( void )
{   restoreirq( hISR.IRQ ); }

/*
  Register Actions can be written as ascii scripts,

  where o and v are hex digits; oo is "offset" and v is value to write:
    "Woo,vvvv;" --> OutWord(base + oo, vvvv);
    "woo,vv;"   --> OutByte(base + oo, vv);
    "Roo;"      --> InWord(base + oo);
    "roo;"      --> InByte(base + oo);
    "F;"        --> not yet implemented for draining 16-bit ADC FIFO
*/
TRegAct ParseRegisterActionToken(char *token)
{
    TRegAct act;
    int count=-1, expectedCount=-2;

    if (token)
    {
        switch (*token)
        {
            case 'w' :
            case 'W' : count = sscanf(token, "%c%02hx,%04x%*[;]", &act.Action, &act.Offset, &act.Value );
                       expectedCount = 3;
                       break;
            case 'r' :
            case 'R' : count = sscanf(token, "%c%02x%*[;]", &act.Action, &act.Offset );
                       expectedCount = 2;
                       break;
            case 'F' : count = 0;
                       expectedCount = 0; //--J2H TODO NYI
                       act.Action = NotApplicable;
                       break;
            default  : act.Action = ERROR_PARSING;
                       break;
        }
    }

    if (count != expectedCount) act.Action = ERROR_PARSING;
    return act;
}



