#include <fstream.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include "testlib.h"
#include "pcilib.h"
#include "card.h"

bool bDemoMode = FALSE;
Word demoDeviceID=0xFFFF;
Word pciVer;
Byte h, highestBusNum;


/*  forward declarations so main() can be ""on top""
 *  Some, most, all of these could/should be refactored into a distinct file, not sure which yet.
 */
void ShowHelp(void);
void ShowClosing( int count );
void PrintPCIBusInfo( Byte h, Word pciVer, Byte highestBusNum );
void PrintProgramHeader( void );
void PrintCardInfo();
void InitVerboseLog();
/* end forward function declarations */

void main( int argc, char* argv[] )
{
    Word i;
    Word index=0;
    Byte busnum, devfunc;
    Word foundcount = 0;
    Word numcards;

    testRun.bDebug = TRUE;
    testRun.bVerbose = TRUE;

    if ((!bDemoMode) && ( !!pci_bios_present( &h, &pciVer, &highestBusNum )))
    {
        errPuts( "No PCI Bus compliant BIOS was found!\r\nAre you running in Windows? This is a DOS-only program.\r\n" );
    }
    else // Yes, PCI BIOS can be called
    {
        InitScreen();
        PrintProgramHeader();
        InitClockHz();
        if (testRun.bVerbose) InitVerboseLog();
        PrintProgramHeader();

        if (!bDemoMode && testRun.bDebug)
            PrintPCIBusInfo( h, pciVer, highestBusNum );

        numcards = InitCardList();
        index = 0;
        i = 0;

		if ( find_pci_device( CardTypeList[i].DeviceID, 0x494f, index, &busnum, &devfunc ) == SUCCESSFUL )
		{
			InitCard( CardTypeList[i], busnum, devfunc ); // --J2H change to "AddNewCard()"
			PrintCardInfo();

			switch ( CardTypeList[i].type )
			{
				case t_PCIeIDIO24: doPCIeIDIO24();
					break;
				default:
					cprintf( "The %d device detected is not recognized by this program.\n""It is either newer than this program, or provided by some other\n""vendor.  Please get the latest version of PCI-/PCIe-Test, and retry.\n", index );
					foundcount--;
					break;
			} // end case
			foundcount++;
			index++; // check at least twice for each card type
		}
		else
		{
			i++;
			index = 0;
		}
	}

	ShowClosing(foundcount);

    txtPuts( "Exiting.\r\n" );
}


void ShowHelp( void )
{
    txtPuts( WHITE, "PCI- / PCIe-Card Test Program\n\r" );
    txtPuts( "--This program tests PCI/PCIe cards and is designed \n\r" );
    txtPuts( "  to run with as few keystrokes as possible.\n\r" );
    txtPuts( "--Place wrap plugs on all ports and install the PCI card.\n\r" );
    txtPuts( "--Run the program by typing 'pcitest' in DOS.\n\r" );
    txtPuts( "--The program detects the card and runs the appropriate tests.\n\r" );
    txtPuts( "--The results of each test are displayed in green or red, green\n\r" );
    txtPuts( "  means pass and red means fail.\n\r" );
    NewLine();
    txtPuts( "--To run debug mode, type 'pcitest /d' at the command prompt.\n\r" );
    txtPuts( "--The colors indicating pass or fail are also present in debug mode.\n\r" );
    txtPuts( "--Debug displays the values generated during each test, which\n\r" );
    txtPuts( "  runs until a key is pressed, moving to the next test.\n\r" );
    txtPuts( "--Press any key to continue test.\n\r" );
    getch();
}   // ShowHelp

void ShowClosing( int count )
{
    Byte b, df;
    gotoxy(1,49);
    if ( count )
    {
        if ( count > 1 ) { txtPuts( "A total of " ); textcolor(WHITE); putch(count+0x30); txtPuts(" cards have been found.\r\n"); }
        else { txtPuts( "One card was found.\r\n" ); }
    }
    else
    {
        txtPuts( "None of the PCI- nor PCIe- cards under consideration have been found.\r\nMake sure your card is installed.\r\n" );

        if ( find_pci_device( 0x9050, 0x10b5, 0, &b, &df ) == SUCCESSFUL ) { txtPuts( YELLOW, "A PLX 9050 card *was* found. Is the EEPROM / Bin missing?\r\n" ); }

        if ( find_pci_device( 0x9030, 0x10b5, 0, &b, &df ) == SUCCESSFUL ) { txtPuts( YELLOW, "A PLX 9030 card *was* found. Is the EEPROM / Bin missing?\r\n" ); }

        if ( find_pci_device( 0x8111, 0x10b5, 0, &b, &df ) == SUCCESSFUL ) { txtPuts( YELLOW, "A PLX 8111 card *was* found. Is the EEPROM / Bin missing?\r\n" ); }
    }
}

void PrintPCIBusInfo( Byte h, Word pciVer, Byte highestBusNum )
{
    char msg[90];
    sprintf(msg, "PCI Bus Found: Hardware Mechanism:% 2x Version % 1x.%02x LastBus:%2x (hex)", h, MSB(pciVer), LSB(pciVer), highestBusNum );
    txtPuts( DARKGRAY, msg );
    NewLine();
}

void PrintProgramHeader( void )
{
    textbackground( BLACK );
    textcolor( LIGHTGRAY );
    clrscr();
    txtPuts( WHITE, "PCI and PCIe Production Test " );

    txtPuts( YELLOW, __DATE__ );
    NewLine();
}

void PrintCardInfo()
{
    txtPuts( WHITE, "%*s", MARGIN_CardNameRight, testRun.theCard.Name );

    txtPuts( DARKGRAY,
             " PID:%04X BUS:%02X DEV_FUNC:%02X BASE:% 4Xh IRQ:%2Xh \n\r",
             testRun.theCard.DeviceID,
             testRun.theCard.bus,
             testRun.theCard.df,
             testRun.theCard.Base,
             testRun.theCard.IRQ
              );

    NewLine();
    testRun.Tests[testRun.LastTest].Top = wherey()+1;
    testRun.Tests[testRun.LastTest].Bottom = wherey()+1;
}

void InitVerboseLog()
{
  VerboseLog.open("log.txt");
}
