#include <fstream.h>
#include <assert.h>
#include <time.h>
#include <stdarg.h>
#include <string.h>
#include "pcilib.h"
#include "testlib.h"
#include "IRQs.h"

#pragma inline

extern void PrintProgramHeader( void );
extern void PrintCardInfo( void );


ofstream VerboseLog;
Byte busnum = 0, devfunc = 0x0;
TMasterTime IRQTime;

int MaxY;


void SeekTestTop( void )
{
    gotoxy( 1, min( MaxY, testRun.Tests[testRun.LastTest].Top ) );
}

void SeekTestBottom( void )
{
    gotoxy( 1, min( MaxY, testRun.Tests[testRun.LastTest].Bottom ) );
}

Byte GetTestBottom( void )
{
    return ( testRun.Tests[testRun.LastTest].Bottom );
}

Byte GetTestTop()
{
    return ( testRun.Tests[testRun.LastTest].Top );
}

void UpdateTestTop()
{
  testRun.Tests[testRun.LastTest].Top = wherey();
  if (testRun.Tests[testRun.LastTest].Top>testRun.Tests[testRun.LastTest].Bottom) testRun.Tests[testRun.LastTest].Bottom = testRun.Tests[testRun.LastTest].Top+1;
}

void UpdateTestBottom( void )
{
    testRun.Tests[testRun.LastTest].Bottom = wherey();
}

void UpdateTestBottomIfLarger( void )
{
    testRun.Tests[testRun.LastTest].Bottom = max( testRun.Tests[testRun.LastTest].Bottom , wherey() );
}

Byte GetSubTestTop()
{
   return ( testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Top );
}

void UpdateSubTestTop()
{
    testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Top = wherey();
}

void SeekSubTestTop()
{
    gotoxy( 1, GetSubTestTop() );
}

Byte GetSubTestBottom()
{
    return GetTestBottom();
}

void UpdateSubTestBottom()
{
    UpdateTestBottom();
}

void SeekSubTestBottom()
{
    SeekTestBottom();
}

void SaveSubTestDataRows( Byte rowsToSave )
{
    testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].ReservedScreenRows = rowsToSave;
}

Word NewLine(void)
{
    txtPuts("\n\r");
    return wherey();
}

void ConClrLine( void ) { cputs( "\r" ); clreol(); }

void clreos( void )
{
    int y = wherey();

    for ( int i = y; i < MaxY; i++ )
    {
        gotoxy( 1, i );
        clreol();
    }

    gotoxy( 1, y );
}

Byte GetTextColor()
{
    struct text_info ti;
    gettextinfo(&ti);
    return ti.normattr;
}

void txtPuts( Byte color, char *msg, ... )
{
    Byte oldColor = GetTextColor();
    char tmp[256];
    va_list arg;
    va_start( arg, msg );
    vsprintf( tmp, msg, arg );
    va_end( arg );

    textcolor( color );
    cputs( tmp );
    textattr( oldColor );
//    if ( testRun.bDebug ) UpdateTestBottom();
}

void txtPuts( char *txt, ... )
{
    char tmp[256];
    va_list ap;
    va_start( ap, txt );
    vsprintf(tmp, txt, ap);
    va_end( ap );
    txtPuts( LIGHTGRAY, tmp );
}

void Banal_txtPuts( char *msg, ... )
{
    Byte oldColor = GetTextColor();
    char tmp[256];
    va_list arg;
    va_start( arg, msg );
    vsprintf( tmp, msg, arg );
    va_end( arg );

    textcolor( LIGHTGRAY );
    cputs( tmp );
    textattr( oldColor );
    if ( testRun.bDebug ) UpdateTestBottom();
}

void errPuts( char *msg) { txtPuts( RED, msg ); }

/*
   prints name of test at bottom of testRun, clears screen below it, animatedly deletes lines between it and previous test name, sets top to previous top + 1, sets bottom to top
*/
void TestAnnounce( char* n, ... )
{
    TTest aTest;

    char msg[256];
    va_list arg;
    va_start(arg, n);
    vsprintf(msg, n, arg); // J2H: Warning! Buffer overrun opportunity!
    va_end(arg);

    if (testRun.bDebug) {SeekTestBottom();NewLine();} else {SeekTestTop();NewLine();}

    aTest.Result = 0; // --J2H todo: change to result enum
    aTest.LastSubTest = 0;
    aTest.Top = GetTestBottom();
    aTest.Bottom = aTest.Top;

    strncpy( aTest.TestName, msg, MAX_TEST_NAME_LENGTH );

    txtPuts( WHITE, "%*s:%*s", MARGIN_TestNameRight, aTest.TestName, MARGIN_TestStatusRight, "running" );

    testRun.Tests[testRun.LastTest] = aTest;
    UpdateTestTop();
    UpdateSubTestTop();

    if ( testRun.bVerbose )
    {
        /*
        int i;

        SeekTestTop(); NewLine();
        for ( i = GetTestTop(); i <= GetTestBottom(); i++ )
        {
            delay(200 / (GetTestBottom() - GetTestTop()));
            delline();
        }
        */
        VerboseLog << testRun.Tests[testRun.LastTest].TestName << endl;
    }

    NewLine();
    clreos();

}

void SubTestAnnounce( char* subTestName, ... )
{
    char msg[256];
    va_list arg;
    va_start(arg, subTestName);
    vsprintf(msg, subTestName, arg); // J2H: Warning! Buffer overrun opportunity!
    va_end(arg);

    SeekSubTestTop(); NewLine();// bottom of previous test/subtest
    testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].ReservedScreenRows = 0;
    testRun.Tests[testRun.LastTest].LastSubTest++;

    strncpy( testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].subTestName, msg, MAX_TEST_NAME_LENGTH );
    testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Result = -4;

    txtPuts( "%*s: %s", MARGIN_SubTestNameRight, testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].subTestName, "Running" );

    if (testRun.bVerbose)
    {
        VerboseLog << testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].subTestName << endl;
    }
    UpdateSubTestTop();
    UpdateSubTestBottom();
    NewLine();
    clreos();
}

int SubTestResult( TResult subResult )
{
    char msg[30] = "";
    TResult subres;

    if ( subResult == DEFAULT_TEST_RESULT )
        subres = testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Result;
    else
    {
        subres = subResult;
        testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Result = subres;
    }

    testRun.Tests[testRun.LastTest].Result += subres; // J2H: Todo: change to enum etc

    SeekSubTestTop();
    txtPuts(BLUE, "%*s", MARGIN_SubTestNameRight, testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].subTestName );

    switch ( subres ) // J2H: Todo: change to enum etc
    {
        case 1:
            strcpy( msg, "Failed" );
            textcolor( LIGHTRED );
            break;

        case 0:
            strcpy( msg, "Passed" );
            textcolor( GREEN );
            break;

        case -1:
            strcpy( msg, "Skipped by User" );
            textcolor( DARKGRAY );
            break;

        case -2:
            strcpy( msg, "Skipped - n/a" );
            textcolor( DARKGRAY );
            break;

        case -3:
            strcpy( msg, "Watch LEDs" );
            textcolor( LIGHTGREEN );
            break;
        case -4:
            strcpy( msg, "Running" );
            textcolor( DARKGRAY );
            break;
    }   // switch

    gotoxy( MARGIN_SubTestNameRight+1, GetSubTestTop() );
    cprintf( ": %*s", MARGIN_SubTestStatusRight, msg );
    clreol();
    gotoxy(1, 1 + wherey());
    UpdateTestBottomIfLarger();
    clreos();
    return subres;
}

int TestResult( int result ) // --J2H todo: Change to enum result type etc
{
    char msg[60] = "";
    char c;
    TResult theResult = 0;

    if ( result == DEFAULT_TEST_RESULT )
        theResult = testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Result;
    else
    {
        theResult = result;
        testRun.Tests[testRun.LastTest].SubTests[testRun.Tests[testRun.LastTest].LastSubTest].Result = theResult;
    }

    testRun.Tests[testRun.LastTest].Result += theResult;

    switch ( theResult ) // J2H: Todo: fix, change to enum etc
    {
        case 1:
            strcpy( msg, "Failed" );
            c = LIGHTRED ;
            break;

        case 0:
            strcpy( msg, "Passed" );
            c = LIGHTGREEN;
            break;

        case -1:
            strcpy( msg, "Skipped by User" );
            c = DARKGRAY;
            break;

        case -2:
            strcpy( msg, "Skipped - n/a" );
            c = DARKGRAY;
            break;

        case -3:
            strcpy( msg, "Watch LEDs" );
            c = LIGHTGREEN;
            break;

        case -4:
            strcpy( msg, "Watch Screen Values" );
            c = LIGHTGREEN;
            break;
    }

    SeekTestTop();
    txtPuts( LIGHTBLUE, "%*s:", MARGIN_TestNameRight, testRun.Tests[testRun.LastTest].TestName );
    txtPuts( c, "%*s", MARGIN_TestStatusRight, msg );
    clreol();

    SeekTestBottom();
    // extra print at test bottom added 2024-03-05 for JM email @9:29
    txtPuts( LIGHTBLUE, "%*s:", MARGIN_TestNameRight, testRun.Tests[testRun.LastTest].TestName );
    txtPuts( c, "%*s", MARGIN_TestStatusRight, msg );
    clreol();

    NewLine;
    UpdateTestBottomIfLarger();
    consumeQueuedKeys();
    testRun.LastTest++;
    return theResult;
}




bool cprintb( uint32 digitalBitsToPrint, Byte NumberOfBitsToPrint, bool bColorAsIfSetIsFail, int cPass, int cFail )
{
    int color;
    int     i;
    uint32  bitMask;
    bool bit;
    // --J2H todo why is the fail color set for the entire set of bits, not on a per-bit basis?
    color = cPass;

    if ( bColorAsIfSetIsFail ) { color = cFail; } // but they wanted it flagged, print "1" in cFail

    for ( i = NumberOfBitsToPrint - 1; i >= 0; i-- )
    {
        bitMask = 1L << i;
        bit = !!( digitalBitsToPrint & bitMask );
        textcolor( color );
        putch( bit ? '1' : '0' );
    }

    clreol();
    return bColorAsIfSetIsFail?1:0;
}   // cprintb


uint32 bitwalk( Byte state, Byte bits ) { return ( uint32 )( 1L << ( uint32 )( state % bits ) ); }

void cputbit( bool bit, bool targetvalue /* =0 */, bool bCompare /* =FALSE */, int targetHitColor /* =WHITE */, int targetMissedColor /* =LIGHTRED */ )
{
    int color = targetHitColor;

    if ( ( bCompare ) && ( bit != targetvalue ) ) { color = targetMissedColor; }

    textcolor( color );
    putch( bit ? '1' : '0' );
}

bool cprintbits( Byte numberOfBitsToPrint, uint32 actualDigitalBits, uint32 targetDigitalBits, bool bCompare /* =FALSE */, uint32 ignoredDigitalBits /* =0 */, int cSame /* = WHITE */, int cDiff /* =LIGHTRED */ )
{
    uint32 diffBits = ( actualDigitalBits ^ targetDigitalBits ) & ~ignoredDigitalBits;
    int i, color;
    uint32 bitMask;

    for ( i = numberOfBitsToPrint - 1; i >= 0; i-- )
    {
        bitMask = bitwalk( i, numberOfBitsToPrint );
        cputbit( actualDigitalBits & bitMask, targetDigitalBits & bitMask, bCompare && ( !!( diffBits & bitMask ) ), cSame, cDiff );
    }

    clreol();
    return ( bCompare && diffBits ) ? 1 : 0;
} // cprintbits with optional bit-wise compare and optional masking

uint32 Debug_PrintBinaryCompare( uint32 d, uint32 c, uint32 m, Byte bits )
{
    uint32  t;  // bits that don't match are set here
    textcolor( LIGHTGRAY );
    cprintf( "%15s", "Expected:" );
    cprintb( d, bits, TRUE, WHITE, LIGHTGREEN );
    textcolor( LIGHTGRAY );
    cprintf( "\n\r%15s", "Read:" );
    cprintb( c, bits, FALSE, WHITE, LIGHTGREEN );
    textcolor( LIGHTGRAY );
    cprintf( "\n\r%15s", "Differences:" );
    cprintb( t = ( d ^ c ) & m, bits, TRUE );
    NewLine();
    return ( t );
}   // Debug_PrintBinaryCompare



Word doRegisterAction(TRegAct Register)
{
    switch(Register.Action)
    {
        case READBYTE:
            Register.Value = InByte( testRun.theCard.Base + Register.Offset );
            break;

        case WRITEBYTE:
            OutByte( testRun.theCard.Base + Register.Offset , Register.Value);
            break;

        case READWORD:
            Register.Value = InWord( testRun.theCard.Base + Register.Offset );
            break;

        case WRITEWORD:
            OutWord( testRun.theCard.Base + Register.Offset , Register.Value );
            break;

        case DRAINFIFO: break;
        case NotApplicable: break;

        case asP104_DIO_48S: // this currently means "clear COS IRQs however this board needs it done"
            {
               Word pA0B0, pC0A1, pB1C1;
               pA0B0 = InWord( testRun.theCard.Base + Register.Offset + 0 );
               pC0A1 = InWord( testRun.theCard.Base + Register.Offset + 2 );
               pB1C1 = InWord( testRun.theCard.Base + Register.Offset + 4 );
               OutWord( testRun.theCard.Base + Register.Offset + 0, pA0B0 );
               OutWord( testRun.theCard.Base + Register.Offset + 2, pC0A1 );
               OutWord( testRun.theCard.Base + Register.Offset + 4, pB1C1 );
            }

            break;
    }
    return Register.Value;
}

void Instruct( char* msg, int c )
{
    txtPuts( c, "%s", msg );
    clreol();
    NewLine();
}

int InstructAndWait( char* msg, int c )
{
	char k;
    int top = wherey();
    consumeQueuedKeys();

    do
    {
        Delay( BLINK_DELAY );
        gotoxy( 1, top );
        Instruct( msg );
        clreol();
        Instruct( "Press a key to proceed" );
        clreol();
        Delay( BLINK_DELAY );
        gotoxy( 1, top );
        Instruct( msg, c );
        clreol();
        Instruct( "Press a key to proceed", GREEN );
        clreol();
    }
    while ( !kbhit() );

    k = getch();
    gotoxy( 1, top );
    clreos(); // erase instruction text
    UpdateTestBottom();
    return k;
}   // InstructAndWait

void consumeQueuedKeys(void)
{
    while (kbhit()) getch(); // consume
}

void InitScreen()
{
    struct text_info ti;

    directvideo=1;
    textcolor(LIGHTGRAY);
    textbackground(BLACK);
    textmode( C4350 );
    clrscr();
    gettextinfo( &ti );
    MaxY = ti.winbottom;
}

void Delay( int ms )
{
    if (ms > 0) delay(ms);
    else switch(ms)
    {
        case DELAY_Machine: delay(1); break;
        case DELAY_Pacing: delay(testRun.bDebug ? 200 : 20); break;
    }
}

void OutWord( Word Address, Word Value )
{
    if ( testRun.bVerbose )
    {
        char msg[30];
        if (!bDemoMode)
            outport( Address, Value );
        sprintf(msg, "%04x=%04x", Address, Value );
        VerboseLog << msg << endl;
    }
    else
    {
        if (!bDemoMode)
            outport( Address, Value );
    }
}

void OutByte( Word Address, Byte Value )
{
    if ( testRun.bVerbose )
    {
        char msg[30];
        if (!bDemoMode)
            outportb( Address, Value );
        sprintf(msg, "%04x=%02x  ", Address, Value );
        VerboseLog << msg << endl;
    }
    else
    {
        if (!bDemoMode)
            outportb( Address, Value );
    }
}

Byte InByte( Word Address )
{
    Byte V;
    if ( testRun.bVerbose )
    {
        char msg[30];
        if (!bDemoMode)
            V = inportb( Address );
        sprintf(msg, "%04x?%02hx  ", Address, V );
        VerboseLog << msg << endl;
    }
    else
    {
        if (!bDemoMode)
            V = inportb( Address );
    }
    return V;
}

Word InWord( Word Address )
{
    Word V;
    if ( testRun.bVerbose )
    {
        char msg[30];
        if (!bDemoMode)
            V = inport( Address );
        sprintf(msg, "%04x?%04x", Address, V );
        VerboseLog << msg << endl;
    }
    else
    {
        if (!bDemoMode)
            V = inport( Address );
    }
    return V;
}

void OutPortDW( Word Port, uint32 Value )
{
    asm mov dx, [Port];
    asm mov eax, [Value];
    asm out dx, eax;

    if ( testRun.bVerbose )
    {
        char msg[30];
        sprintf(msg, "%04x=%08lx  ", Port, Value );
        VerboseLog << msg << endl;
    }
}   // OutPortDW

uint32 InPortDW( Word Port )
{
    uint32 result;
    asm mov dx, [Port];
    asm in  eax, dx;
    result = _EAX;

    if ( testRun.bVerbose )
    {
        char msg[30];
        sprintf(msg, "%04x?%08lx  ", Port, result );
        VerboseLog << msg << endl;
    }
    return ( result );
}   // InPortDW

